//------------------------------------------------------------------------------
//
//  scene.cpp
//
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//  include
//------------------------------------------------------------------------------
#include "common.h"

static LPDIRECT3DPIXELSHADER9   pshader    = NULL;
static LPDIRECT3DPIXELSHADER9   pshader_f  = NULL;
static LPDIRECT3DTEXTURE9       texture0   = NULL;
static LPDIRECT3DSURFACE9       surface0   = NULL;
static LPDIRECT3DTEXTURE9       objtex     = NULL;
static LPDIRECT3DSURFACE9       objsurface = NULL;
static LPDIRECT3DSURFACE9       backbuffer = NULL;
static LPCSTR filename = TEXT("main.fx");
static LPCSTR str_ps   = "ps_main";
static LPCSTR str_ps_f = "ps_main_f";

#define TEX_XY   256

//shader constant
static D3DXVECTOR4 shader_const[256];

//------------------------------------------------------------------------------
//  variable
//------------------------------------------------------------------------------
struct d3dverts {
  FLOAT x, y, z, w;
  FLOAT u, v;
  enum { fvf = D3DFVF_XYZRHW | D3DFVF_TEX1 };
};

static const d3dverts quadverts[4] = {
 { -0.5+      0,  -0.5+      0, 0.0f, 1.0f, 0.0f, 0.0f},
 { -0.5+ScreenX,  -0.5+      0, 0.0f, 1.0f, 1.0f, 0.0f},
 { -0.5+      0,  -0.5+ScreenY, 0.0f, 1.0f, 0.0f, 1.0f},
 { -0.5+ScreenX,  -0.5+ScreenY, 0.0f, 1.0f, 1.0f, 1.0f},
};

//------------------------------------------------------------------------------
//
//  function definition
//
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//  InitScene
//------------------------------------------------------------------------------
void InitScene() {
  HRESULT hRet = E_FAIL;
  LPD3DXBUFFER tmp;
  LPD3DXBUFFER err;
  TermScene();
  
  hRet = D3DXCompileShaderFromFile( filename,  0, 0, str_ps, "ps_3_0", D3DXSHADER_OPTIMIZATION_LEVEL3|D3DXSHADER_PREFER_FLOW_CONTROL, &tmp, &err, 0 );
  if(hRet != D3D_OK) {
    printf("Error : %s\n", err->GetBufferPointer());
    return;
  }
  hRet = d3ddevice->CreatePixelShader( (DWORD*)tmp->GetBufferPointer(), &pshader );
  if(hRet != D3D_OK) {
    printf("Error : %s\n", err->GetBufferPointer());
    return;
  }
  RELEASE(tmp);
  
  hRet = D3DXCompileShaderFromFile( filename,  0, 0, str_ps_f, "ps_3_0", D3DXSHADER_OPTIMIZATION_LEVEL3|D3DXSHADER_PREFER_FLOW_CONTROL, &tmp, &err, 0 );
  if(hRet != D3D_OK) {
    printf("Error : %s\n", err->GetBufferPointer());
    return;
  }
  hRet = d3ddevice->CreatePixelShader( (DWORD*)tmp->GetBufferPointer(), &pshader_f );
  if(hRet != D3D_OK) {
    printf("Error : %s\n", err->GetBufferPointer());
    return;
  }
  RELEASE(tmp);
  printf("Load Shader.\n");
  
  d3ddevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &backbuffer);
  d3ddevice->CreateTexture(ScreenX, ScreenY, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A32B32G32R32F, D3DPOOL_DEFAULT, &texture0, 0);
  texture0->GetSurfaceLevel(0, &surface0);
  d3ddevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
  d3ddevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
  d3ddevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
  
  d3ddevice->CreateTexture(TEX_XY, TEX_XY, 1, D3DUSAGE_DYNAMIC, D3DFMT_A32B32G32R32F, D3DPOOL_DEFAULT, &objtex, 0);
  objtex->GetSurfaceLevel(0, &objsurface);
  d3ddevice->SetTexture(1, objtex );
  d3ddevice->SetSamplerState(1, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
  d3ddevice->SetSamplerState(1, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
  d3ddevice->SetSamplerState(1, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
  
  d3ddevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
  d3ddevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
}

//------------------------------------------------------------------------------
//  TermScene
//------------------------------------------------------------------------------
void TermScene() {
  RELEASE(objsurface);
  RELEASE(objtex);
  RELEASE(texture0);
  RELEASE(surface0);
  RELEASE(backbuffer);
  RELEASE(pshader);
  RELEASE(pshader_f);
}

int random() {
  static int a = 1, b = 234567, c = 890123;
  a += b; b += c; c += a;
  return (a >> 16);
}

float frand() {
  return (float)random() / 32768.0f;
}

//------------------------------------------------------------------------------
//  DoScene
//------------------------------------------------------------------------------
void DoScene() {
  static unsigned long t = timeGetTime();
  unsigned long tt = timeGetTime();
  float delta = (float)(tt - t) / 1000.0f;
  float k = 0;
  k = shader_const[0].x;
  t = tt;
  if(GetAsyncKeyState(VK_F5)) {
    InitScene();
  }
  
  if(objsurface) {
    D3DLOCKED_RECT lockedRect;
    objsurface->LockRect(&lockedRect, 0, 0);
    D3DXVECTOR4 *d = (D3DXVECTOR4 *)lockedRect.pBits;
    ZeroMemory(d, sizeof(D3DXVECTOR4) * TEX_XY * TEX_XY);
    for(int i =  0 ; i < 1000; i++) {
      float x = sin(0.1*k + i * 0.3) * 3;
      float y = cos(0.1*k + i * 0.5) * 4;
      unsigned int  ix = unsigned int((TEX_XY/2) + ((TEX_XY/2)*x)) % TEX_XY;
      unsigned int  iy = unsigned int((TEX_XY/2) + ((TEX_XY/2)*y)) % TEX_XY;
      d[ix + iy * TEX_XY].x = 1;
      d[ix + iy * TEX_XY].y = 1;
      d[ix + iy * TEX_XY].z = 1;
    }
    objsurface->UnlockRect();
  }
  
  if(d3ddevice->BeginScene() == D3D_OK) {
    if(pshader) {
      d3ddevice->SetPixelShaderConstantF(0, (FLOAT*)&shader_const, 64);
      d3ddevice->SetPixelShader( pshader );
    } else {
      printf("Error : Pshader\n");
    }

    shader_const[0].x += delta;
    d3ddevice->SetRenderTarget(0, surface0);
    d3ddevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, 0, 1.0f, 0);
    d3ddevice->SetFVF( d3dverts::fvf  );
    d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, quadverts, sizeof(d3dverts));

    d3ddevice->SetRenderTarget(0, backbuffer);
    d3ddevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER|D3DCLEAR_STENCIL, 0, 1.0f, 0);
    d3ddevice->SetFVF( d3dverts::fvf  );
    d3ddevice->SetTexture(0, texture0 );
    d3ddevice->SetPixelShader( pshader_f );
    d3ddevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, quadverts, sizeof(d3dverts));
    d3ddevice->EndScene();
  }
  d3ddevice->Present( NULL, NULL, NULL, NULL );
}

